from kubiat.mongodb import put_item_impl, put_items_impl
from kubiat.mongodb import get_item_impl, get_items_impl
from kubiat.mongodb import get_item_not_id_impl
from kubiat.mongodb import update_item_impl, update_item_not_id_impl
from constants import _DB, _ID


# returns id
def put_item(item):
    return put_item_impl(_DB, _ID, item)


# returns id list
def put_items(items):
    return put_items_impl(_DB, _ID, items)


def get_item(key):
    return get_item_impl(_DB, {_ID: key})


def get_items(list_key_ids):
    return get_items_impl(_DB, _ID, list_key_ids)


def get_item_not_id(key):
    return get_item_not_id_impl(_DB, {_ID: key})


# attr_dict is {attr_name_1:attr_value_1, ...}
def update_item(key, attr_dict):
    return update_item_impl(_DB, {_ID:  key}, attr_dict)


# attr_dict is {attr_name_1:attr_value_1, ...}
def update_item_not_id(key, attr_dict):
    return update_item_not_id_impl(_DB, {_ID: key}, attr_dict)



""" connection to mongodb """
import pymongo
from bson.objectid import ObjectId
import time
import os
from kubiat.service import APP
import constants

app_space = ''
if 'KUBIAT' in os.environ:
    app_space = os.environ['KUBIAT']
else:
    APP.logger.error('No App Space defined')
mongo_db_name = app_space

mongo_db_ip = 'kubiat-mongodb.mongodb'
if 'TEST' in os.environ:
    mongo_db_ip = 'localhost'

APP.logger.info('Running with IP: '+str(mongo_db_ip))
APP.logger.info('        with DB: '+constants._DB
                + ' ('+constants._ID+') on '+str(mongo_db_name))

myclient = pymongo.MongoClient('mongodb://root:admin@'+mongo_db_ip+':27017/')
mydb = myclient[mongo_db_name]


def convertTo_mid(key):
    mid_name = list(key.keys())[0]
    return {'_id': ObjectId(key[mid_name])}


def put_item_impl(table_name, key_name, item):
    item_copy = item.copy()
    item_copy['created_at'] = round(time.time())

    mycol = mydb[table_name]
    ret = mycol.insert_one(item_copy)
    item_copy[key_name] = str(item_copy.pop('_id'))
    return str(ret.inserted_id)


def put_items_impl(table_name, key_name, items):
    for item in items:
        item['created_at'] = round(time.time())

    mycol = mydb[table_name]
    ret = mycol.insert_many(items)
    for item in items:
        item[key_name] = str(item.pop('_id'))
    idlist = []
    for i in ret.inserted_ids:
        idlist.append(str(i))
    return idlist


def get_item_impl(table_name, key):
    item = mydb[table_name].find_one(convertTo_mid(key))
    if (item):
        item[list(key.keys())[0]] = str(item.pop('_id'))
    return item


def get_item_not_id_impl(table_name, key):
    item = mydb[table_name].find_one(key)
    if (item):
        item.pop('_id')
    return item


def get_items_impl(table_name, key_name, list_key_ids):
    items = []
    for orig_id in list_key_ids:
        item = get_item_impl(table_name, {key_name: orig_id})
        items.append(item)
    return items


# attr_dict is {attr_name_1:attr_value_1, ...}
def update_item_impl(table_name, key, attr_dict):
    attr_dict['last_updated'] = round(time.time())
    newvalues = {"$set": attr_dict}
    mydb[table_name].update_one(convertTo_mid(key), newvalues)
    return


def update_item_not_id_impl(table_name, key, attr_dict):
    attr_dict['last_updated'] = round(time.time())
    newvalues = {"$set": attr_dict}
    mydb[table_name].update_one(key, newvalues)
    return



import requests
import os
from cloudevents.http import CloudEvent, to_structured
from kubiat.service import APP

app_space = ''
if 'KUBIAT' in os.environ:
    app_space = os.environ['KUBIAT']
else:
    APP.logger.error('No App Space defined')


def callService(myfn, data):
    dest = "https://"+myfn+"."+app_space+".kubiat.eu/"
    attributes = {"type": "direct-call", "source": "empty"}
    return sendEvent(dest, data, attributes)


def callBroker(data, attributes):
    broker = ("http://broker-ingress.knative-eventing.svc.cluster.local/"
              + app_space
              + "/default")
    return sendEvent(broker, data, attributes)


def sendEvent(dest, data, attributes):
    event = CloudEvent(attributes, data)
    headers, body = to_structured(event)
    r = requests.post(url=dest, data=body, headers=headers)
    if (r is not None and r.ok and r.status_code == 200):
        return r.json()
    if (r.status_code != 202):
        APP.logger.error('unexpected response ' + str(r.status_code) + r.text)
    return {}



import logging
from flask import Flask

APP = Flask(__name__)
if __name__ != '__main__':
    gunicorn_logger = logging.getLogger('gunicorn.error')
    APP.logger.handlers = gunicorn_logger.handlers
    APP.logger.setLevel(gunicorn_logger.level)
